﻿#undef SINGLE_TEST

// Define INGAME_ZOOM to allow zooming in and out with
// the PageUp and PageDown keys.
#define INGAME_ZOOM

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace CarFire
{
    /// <summary>
    /// This class is responsible for controlling what draws to the screen when the game is running.
    /// </summary>
    public class Display
    {
        bool playerChosen = false;
        List<Projectile> mProjectiles = new List<Projectile>();
        //List<IPlayer> mCharacters = new List<IPlayer>();
        //IPlayer[] mCharacters = new IPlayer[4];
        Texture2D everything;
        Texture2D projectile1;
        Game mGame;
#if SINGLE_TEST
        List<Keys> mLastPressedKeys = new List<Keys>();
#endif
        public Display(Game game)
        {
            mGame = game;
            /*
            mMap = aMap;
            mCharacters = characters;
             */
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        public void LoadContent(ContentManager contentManager)
        {
            everything = contentManager.Load<Texture2D>("cs");
            projectile1 = contentManager.Load<Texture2D>("projectile");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        public void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public void Update(TimeSpan timespan, GameState state, int thisPlayer)
        {
            
            //INPUT - testing input... has to be through network later
#if SINGLE_TEST
            KeyboardState keyState = Keyboard.GetState();

            List<Keys> pressedKeys = new List<Keys>();
            List<Keys> releasedKeys = new List<Keys>();

            Keys[] pressedKeysArray = keyState.GetPressedKeys();
            foreach (Keys k in pressedKeysArray)
            {
                if (!mLastPressedKeys.Contains(k)) pressedKeys.Add(k);
                else mLastPressedKeys.Remove(k);
            }

            releasedKeys = mLastPressedKeys;
            mLastPressedKeys = new List<Keys>(pressedKeysArray);
            //Just apply input for the first player
            mCharacters[0].MovePlayer(pressedKeys);
            if (pressedKeys.Contains(Keys.Enter) && !releasedKeys.Contains(Keys.Enter))
            {
                mProjectiles.Add(new Projectile(mMap, everything, new Vector2(-5,0), mCharacters[0].GridX +1, mCharacters[0].GridY+1, (int)mCharacters[0].GridX*(int)Map.PixelsToUnitSquares,(int)mCharacters[0].GridY*(int)Map.PixelsToUnitSquares));
            }
            mMap.CenterCell = new Vector2(mCharacters[0].GridX, mCharacters[0].GridY);
#endif
            
            //Handle projectiles - update and check for wall collisions
            for (int i = 0; i < mProjectiles.Count; i++ )
            {
                bool removed = false;
                if (!mGame.State.Map.IsCellOpen(new Point(mProjectiles[i].Coordinates.X, mProjectiles[i].Coordinates.Y)))
                {
               
                    mProjectiles.RemoveAt(i);
                    removed = true;
                    i--;
                }
                if(!removed)
                mProjectiles[i].Update(timespan);
                
            }
            //Check for collisons
            for (int j = 0; j < mGame.State.mCharacters.Length; j++) 
            {
               
                if(mGame.State.mCharacters[j] != null)
                for (int i = 0; i < mProjectiles.Count; i++)
                {
                    if (mProjectiles[i].Coordinates.X == mGame.State.mCharacters[j].Coordinates.X && mProjectiles[i].Coordinates.Y == mGame.State.mCharacters[j].Coordinates.Y)
                    {
                        mGame.State.mCharacters[j].causeDamageTo(mProjectiles[i].Damage);
                        Console.WriteLine(mGame.State.mCharacters[j].Health);
                        mProjectiles.RemoveAt(i);
                        i--;
                    }
                }
            }
            //Update input for each player
            for (int i = 0; i < mGame.State.NumberOfPlayers; i++)
            {
                //If player has not selected a player yet let them select one.
                if (mGame.State.mCharacters[i] == null)
                {
                    if (mGame.State.GetKeysDown(i).Contains(Keys.Enter))
                    {
                        //mCharacters[i] = new Human(mGame, "", everything, projectile1, this, mGame.State.Map.GetStartingPositionForPlayer(i + 1));
                    }
                }
                //Regular player input updates
                else
                {

                    mGame.State.mCharacters[i].UpdateInput(timespan, mGame.State.GetKeysDown(i));
                   
                }
            }
            if (mGame.State.mCharacters[thisPlayer] != null)
            {
                mGame.State.Map.CenterCell = mGame.State.mCharacters[thisPlayer].Position;
            }
                //Handle wall collisions of projectiles again...
            for (int i = 0; i < mProjectiles.Count; i++)
            {
                if (!mGame.State.Map.IsCellOpen(new Point(mProjectiles[i].Coordinates.X, mProjectiles[i].Coordinates.Y)))
                {
                    mProjectiles.RemoveAt(i);
                    i--;
                }          

            }

#if INGAME_ZOOM
            if (Keyboard.GetState().IsKeyDown(Keys.PageUp)) mGame.State.Map.Zoom = mGame.State.Map.Zoom + 0.5f;
            if (Keyboard.GetState().IsKeyDown(Keys.PageDown)) mGame.State.Map.Zoom = mGame.State.Map.Zoom - 0.5f;
#endif
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="spriteBatch">Used to draw with</param>
        public void Draw(SpriteBatch spriteBatch)
        {
            mGame.State.Map.Draw(spriteBatch);
            mGame.State.Entities.ForEach(delegate(IEntity e) { e.Draw(spriteBatch); });

            foreach(Projectile projectile in mProjectiles)
            {
                projectile.Draw(spriteBatch);
            }
            for(int i = 0; i < mGame.State.NumberOfPlayers; i++)//IPlayer character in mCharacters)
            {

                if (mGame.State.mCharacters[i] != null)
                {
                    mGame.State.mCharacters[i].Draw(spriteBatch);
                    
                }
            }
            
        }
        /// <summary>
        /// Add a projectile to the Display.
        /// </summary>
        /// <param name="projectile"></param>
        public void AddProjectiles(Projectile projectile)
        {
            mProjectiles.Add(projectile);
        }
    }
}

