﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace CarFire
{
    /// <summary>
    /// A class to represent any object that can exist as an
    /// independent piece of the game.
    /// </summary>
    public interface IEntity
    {
        /// <summary>
        /// Load the resources the entity needs.
        /// </summary>
        /// <param name="contentManager">The foobar.</param>
        void LoadContent(ContentManager contentManager);

        /// <summary>
        /// Update the entity's state.
        /// </summary>
        /// <param name="timeSpan">The timeslice.</param>
        void Update(TimeSpan timeSpan);

        /// <summary>
        /// Render the entity on the screen.
        /// </summary>
        /// <param name="spriteBatch">The widget.</param>
        void Draw(SpriteBatch spriteBatch);

        /// <summary>
        /// Get the actual position.
        /// </summary>
        Vector2 Position { get; }

        /// <summary>
        /// Get the coordinates on the grid.
        /// </summary>
        Point Coordinates { get; }
    }
}
